/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import UIKit

@objc class DeviceDetailView: UIViewController, UITextViewDelegate {
    
    // Needed by the loops in the event handler button actions below.
    @objc var timer1: Timer?
    
    @objc var stopped:Bool = false
    
    //MARK: Properties
    
    @IBOutlet weak var headLineLabelOutlet: UILabel! {
        didSet {
            headLineLabelOutlet.font = UIFont(name: (headLineLabelOutlet.font?.fontName)!,
                                              size: (headLineLabelOutlet.font?.pointSize)! * 1.2)
        }
    }

    @IBOutlet weak var currentValueLabelOutlet: UILabel!

    @IBOutlet weak var stopButtonOutlet: UIButton! {
        didSet {
            stopButtonOutlet.layer.cornerRadius = 6.0
            stopButtonOutlet.layer.borderColor = UIColor.lightGray.cgColor
            stopButtonOutlet.layer.borderWidth = 1.0
            stopButtonOutlet.layer.backgroundColor = UIColor(white: 0.90,
                                                             alpha: 1.0).cgColor
        }
    }
    
    @IBOutlet weak var resetButtonOutlet: UIButton! {
        didSet {
            resetButtonOutlet.layer.cornerRadius = 6.0
            resetButtonOutlet.layer.borderColor = UIColor.lightGray.cgColor
            resetButtonOutlet.layer.borderWidth = 1.0
            resetButtonOutlet.layer.backgroundColor = UIColor(white: 0.90,
                                                              alpha: 1.0).cgColor
            if Provisioning.isProvisioningStatic() {
                resetButtonOutlet.isHidden = true
            }
        }
    }
    

    @IBOutlet weak var outputLabelOutlet: UILabel!
    
    @IBOutlet weak var outputScrollViewOutlet: UIScrollView! {
        didSet {
            outputScrollViewOutlet.layer.cornerRadius = 6.0
            outputScrollViewOutlet.layer.borderColor = UIColor.lightGray.cgColor
            outputScrollViewOutlet.layer.borderWidth = 1.0
            outputScrollViewOutlet.layer.backgroundColor = UIColor(white: 0.90,
                                                                   alpha: 1.0).cgColor
        }
    }
    
    @IBOutlet weak var outputTextViewOutlet: UITextView! {
        didSet {
            outputTextViewOutlet.layer.cornerRadius = 6.0
            outputTextViewOutlet.layer.borderColor = UIColor.lightGray.cgColor
            outputTextViewOutlet.layer.borderWidth = 1.0
            outputTextViewOutlet.layer.backgroundColor = UIColor(white: 0.90,
                                                                 alpha: 1.0).cgColor
        }
    }

    @IBOutlet var ViewControllerOutlet: UIView! {
        didSet {
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        outputTextViewOutlet.delegate = self
        UIGraphicsBeginImageContext(CGSize(width: self.view.frame.size.width,
                                           height: self.view.frame.size.width - 18))
        UIImage(named: "cloudes-sw.png")?.drawAsPattern(in: self.view.bounds)
        let image : UIImage = UIGraphicsGetImageFromCurrentImageContext()!
        UIGraphicsEndImageContext()
        ViewControllerOutlet.backgroundColor = UIColor(patternImage: image)
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        do {
            try self.dataConnector = DataConnector()
        
        } catch {
            showError(controller: self, alertTitle: "Provisioning Failure",
                      message: kProvisioningFailure, actionTitle: "OK",
                      handler: { (action: UIAlertAction?) -> () in
                // Ensure that user can re-provision the device
                removeLastSettingsFile()
                self.navigationController?.popToRootViewController(animated: false)
            })
        }
        
        if dataConnector != nil {
            //self.navigationItem.setHidesBackButton(true, animated: true)
        
            headLineLabelOutlet.text = kGWwvHeadline
            stopButtonOutlet.setTitle(kStop, for: UIControlState())
            // TODO unhide exit button
            stopButtonOutlet.isHidden = true
            resetButtonOutlet.setTitle(kReset, for: UIControlState())
            outputLabelOutlet.text = kOutput
            outputTextViewOutlet.text = kNotAttached
            outputTextViewOutlet.isScrollEnabled = true
            outputTextViewOutlet.isEditable = false
            currentValueLabelOutlet.textAlignment = .left
            writeDummyOutput()
            
            // Prepare and activate the gateway device
            dataConnector!.pgd(callback: { error in
                if error == nil {
                    if !self.dataConnector!.isDeviceInitialized() {
                        showError(controller: self,
                                  alertTitle: "Device Initialization Alert",
                                  message: self.dataConnector!.getErrorMessage(),
                                  actionTitle: "Exit App",
                          handler: { (action: UIAlertAction?) -> () in
                            exit(0)})
                    } else {
                        self.stopped = false
                        self.dataConnector!.unstop()
                        // Loop for sample measurements
                        // Timers must be started on the main thread.
                        // This code is executed from the Alamofire response
                        // queue. This queue has been changed to be NOT the main
                        // queue.
                        DispatchQueue.main.async(execute: {
                            self.timer1 = Timer.scheduledTimer(timeInterval: 5.0,
                                target: self,
                                selector: #selector(DeviceDetailView.updateData),
                                userInfo: nil, repeats: true)
                            // Create and manage the indirectly connected devices.
                            self.dataConnector!.startSample()
                        })
                    }
                }
            })
        }
        
    }
    
    override func viewDidAppear(_ animated: Bool) {
        if self.dataConnector != nil {
            self.navigationItem.setHidesBackButton(true, animated: false)
        }
        super.viewDidAppear(animated)
    }
    
    @objc func writeDummyOutput() -> Void {

        let attrsBold = [NSAttributedStringKey.font : UIFont.boldSystemFont(ofSize: 17)]
        let attrsNormal = [NSAttributedStringKey.font : UIFont.systemFont(ofSize: 15)]
        
        let boldText = "Humidity Sensor <id>\n"
        let attributedString = NSMutableAttributedString(string: boldText,
                                                         attributes: attrsBold)
        let normalText = kCurrentValue + "  %\n" + kHMaxThreshold + " %\n"
        let normalString = NSMutableAttributedString(string: normalText,
                                                     attributes: attrsNormal)
        let boldText2 = "Temperature Sensor <id>\n"
        let boldString2 = NSMutableAttributedString(string: boldText2,
                                                    attributes: attrsBold)
        let normalText2 = kCurrentValue + " °C\n" +
            kTMinMaxTemp + " °C\n" +
            kTThreshold + " °C\n" +
            kTStartTime
        
        let normalString2 = NSMutableAttributedString(string: normalText2,
                                                      attributes: attrsNormal)
        
        attributedString.append(normalString)
        attributedString.append(boldString2)
        attributedString.append(normalString2)
        currentValueLabelOutlet.attributedText = attributedString
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // Close the data connector and all used resources, 
    // gateway device, tam and communication channel
    @objc func closeDataConnector() {
        self.dataConnector!.close()
    }
    // TODO: Hack to not crash when app is closed
    // Need to find out how this worked before the ChooserProvisioner screen
    // Since DeviceDetailView has never been the root screen
    static var _dataConnector: DataConnector!
    @objc static func closeDataConnector() {
        _dataConnector?.close()
    }
    
    var dataConnector: DataConnector? = nil
    
    @objc func askSensor() {
        if(!self.stopped) {
            
            let attrsBold = [NSAttributedStringKey.font : UIFont.boldSystemFont(ofSize: 17)]
            let attrsNormal = [NSAttributedStringKey.font : UIFont.systemFont(ofSize: 15)]
            
            let boldText = "Humidity Sensor " +
                self.dataConnector!.getEndpointIdH() + "\n"
            let attributedString = NSMutableAttributedString(string: boldText,
                                                             attributes: attrsBold)

            let normalText = kCurrentValue + " " +
                self.dataConnector!.getHumidity().description + " %\n" +
                kHMaxThreshold + " " +
                self.dataConnector!.getMaxThreshold().description + " %\n"
            let normalString = NSMutableAttributedString(string: normalText,
                                                         attributes: attrsNormal)
            
            let boldText2 = "Temperature Sensor " +
                self.dataConnector!.getEndpointIdT() + "\n"
            let boldString2 = NSMutableAttributedString(string: boldText2,
                                                        attributes: attrsBold)
            
            let sTime = Double(self.dataConnector!.getStartTime())
            let theDate = Date(timeIntervalSince1970: sTime)
            let dateFormatter = DateFormatter()
            dateFormatter.dateStyle = .short
            dateFormatter.timeStyle = .short
            let lastOn = dateFormatter.string(from: theDate)
            
            let normalText2 = kCurrentValue + " " +
                String(format: "%.2f", self.dataConnector!.getTemperature()) + " °C\n" +
                kTMinMaxTemp + " " +
                String(format: "%.2f", self.dataConnector!.getMinTemperature()) + " / " +
                String(format: "%.2f", self.dataConnector!.getMaxTemperature()) + " °C\n" +
                kTThreshold + " " +
                String(format: "%d", self.dataConnector!.getThermoMinThreshold()) + " / " +
                String(format: "%d", self.dataConnector!.getThermoMaxThreshold()) + " °C\n" +
                kTStartTime + " " + lastOn
            let normalString2 = NSMutableAttributedString(string: normalText2,
                                                          attributes: attrsNormal)
            attributedString.append(normalString)
            attributedString.append(boldString2)
            attributedString.append(normalString2)
            currentValueLabelOutlet.attributedText = attributedString
            
            let str: String = self.dataConnector!.getOutputText()
            self.outputTextViewOutlet.text = str
            // Force text field to (invisibly) scroll until latest line is at bottom
            let bottom: NSRange = NSMakeRange(self.outputTextViewOutlet.text.count - 10, 1)
            self.outputTextViewOutlet.scrollRangeToVisible((bottom))
        }
    }
    
    @objc func updateData() {
        self.askSensor()
    }
    
    // MARK: Actions
    
    // Action for the button "Reset"
    @IBAction func onResetButton(_ sender: UIButton) {
        self.dataConnector!.stop()
        self.stopped = true
        // stop the loops
        self.timer1?.invalidate()
        // self.timer2?.invalidate()
        outputTextViewOutlet.text = kNotAttached
        writeDummyOutput()
        // Allow reprovisioning
        removeLastSettingsFile()
        self.navigationController?.popToRootViewController(animated: false)
        
    }
    
    // Action for the button "Stop"
    @IBAction func onStopButton(_ sender: UIButton) {
        self.dataConnector!.stop()
        self.stopped = true
        
        // stop the loops
        self.timer1?.invalidate()

        outputTextViewOutlet.text = kNotAttached
        writeDummyOutput()
        saveSettingsFile(path: URL(fileURLWithPath: Provisioning.getProvisioningFilePath()).lastPathComponent,
            password: String(Provisioning.getProvisioningPassword()))
        exit(0)
    }
}
