/*
 *  Copyright © 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 *  This software is dual-licensed to you under the MIT License (MIT) and
 *  the Universal Permissive License (UPL). See the LICENSE file in the root
 *  directory for license terms. You may choose either license, or both.
 */

import Foundation

public class HumiditySensor {
    
    private var hmin: Int = 0
    private var hmax: Int = 100
    
    private var x: Double = 0
    private var amplitude: Double
    private var setPoint: Double
    private var maxThreshold: Int
    private var hardwareId: String
    private var persistentData : UserDefaults
    
    public init(id: String) {
        hardwareId = id
        
        persistentData = UserDefaults.standard
        if (persistentData.object(forKey: hardwareId + "maxThreshold") != nil) {
            maxThreshold = persistentData.integer(forKey: hardwareId + "maxThreshold")
        } else {
            maxThreshold = 80
            persistentData.set(maxThreshold, forKey: hardwareId + "maxThreshold")
        }
        
        setPoint = Double(maxThreshold) * 0.9
        amplitude = Double(maxThreshold) - setPoint + 0.5
    }
    
    public func getHumidity() -> Int {
        let delta: Double = self.amplitude * Double(sin(Double(self.x) * Double.pi / 180))
        self.x = self.x + 30
        return Int(round(self.setPoint + delta))
    }
    
    public func setMaxThreshold(threshold: Int) {
        self.maxThreshold = threshold
        persistentData.set(threshold, forKey: hardwareId + "maxThreshold")
        self.setPoint = Double(self.maxThreshold) * 0.9
        self.amplitude = Double(self.maxThreshold) - self.setPoint + 0.5
    }
    
    public func getMaxThreshold() -> Int {
        return self.maxThreshold
    }
    
    public func getManufacturer() -> String {
        return "Sample"
    }
    
    public func getModelNumber() -> String {
        return "MN-" + self.hardwareId
    }
    
    public func getSerialNumber() -> String {
        return "SN-" + self.hardwareId
    }
    
    public func getHardwareId() -> String {
        return self.hardwareId
    }
}
